package cn.wzpmc.entities;

import cn.wzpmc.configuration.GuiConfiguration;
import cn.wzpmc.mixin.SoundManagerMixin;
import cn.wzpmc.mixin.SoundSystemMixin;
import cn.wzpmc.sounds.MusicSoundInstance;
import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.client.MinecraftClient;
import net.minecraft.client.sound.SoundManager;
import net.minecraft.client.texture.NativeImage;
import net.minecraft.client.texture.NativeImageBackedTexture;
import net.minecraft.util.Identifier;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.List;
import java.util.*;

@Environment(EnvType.CLIENT)
public class SongObject {
    private final String songName;
    private final MusicSoundInstance soundInstance;
    private final String artists;
    private final Identifier imageId;
    private final LinkedHashMap<Long, String> lyrics = new LinkedHashMap<>();
    private final long totalDuringTime;
    private final MinecraftClient minecraftClient;
    private long nowSongDuringTime;
    private int startTick = -1;

    public SongObject(MinecraftClient minecraftClient, String songName, String artists, long totalDuringTime, byte[] picByteArray, byte[] songByteArray, String lyrics) throws IOException {
        this.minecraftClient = minecraftClient;
        this.songName = songName;
        this.soundInstance = new MusicSoundInstance(minecraftClient.player, new ByteArrayInputStream(songByteArray), this);
        this.artists = artists;
        this.nowSongDuringTime = 0;
        this.totalDuringTime = totalDuringTime;
        // 图片处理部分
        BufferedImage originalImage = ImageIO.read(new ByteArrayInputStream(picByteArray));
        BufferedImage resizedImage = new BufferedImage(GuiConfiguration.coverWidth, GuiConfiguration.coverHeight, BufferedImage.TYPE_INT_RGB);
        Graphics2D g2d = resizedImage.createGraphics();
        g2d.drawImage(originalImage, 0, 0, GuiConfiguration.coverWidth, GuiConfiguration.coverHeight, null);
        g2d.dispose();
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        ImageIO.write(resizedImage, "png", byteArrayOutputStream);
        byte[] byteArray = byteArrayOutputStream.toByteArray();
        NativeImage image = NativeImage.read(new ByteArrayInputStream(byteArray));
        NativeImageBackedTexture backedTexture = new NativeImageBackedTexture(image);
        this.imageId = minecraftClient.getTextureManager().registerDynamicTexture("der", backedTexture);
        // 歌词处理部分
        String[] lineSplit = lyrics.split("\n");
        for (String fileLine : lineSplit) {
            String[] timeLrcSplit = fileLine.split("]");
            String time = timeLrcSplit[0].replace("[", "");
            String[] msSplit = time.split("\\.");
            String[] minSecSplit = msSplit[0].split(":");
            long min = Long.parseLong(minSecSplit[0]);
            long sec = Long.parseLong(minSecSplit[1]);
            long ms = Long.parseLong(msSplit[1]);
            StringBuilder line = new StringBuilder();
            for (int i = 1; i < timeLrcSplit.length; i++) {
                line.append(timeLrcSplit[i]);
            }
            long lrcTime = (min * 60 + sec) * 1000 + ms;
            this.lyrics.put(lrcTime, line.toString());
        }
    }

    public void playSong() {
        SoundManager soundManager = this.minecraftClient.getSoundManager();
        soundManager.playNextTick(this.soundInstance);
    }

    public Identifier getImageId() {
        return this.imageId;
    }

    public String getName() {
        return this.songName;
    }

    public String getArtists() {
        return this.artists;
    }

    public Long getNowDuringTime() {
        SoundManagerMixin soundManager = (SoundManagerMixin) this.minecraftClient.getSoundManager();
        SoundSystemMixin soundSystem = (SoundSystemMixin) soundManager.getSoundSystem();
        if (startTick == -1 && soundSystem.getSoundEndTick().containsKey(this.soundInstance)) {
            startTick = soundSystem.getTicks();
        }
        int ticks = soundSystem.getTicks() - startTick;
        this.nowSongDuringTime = ticks * 50L;
        return this.nowSongDuringTime;
    }

    public Long getTotalDuringTime() {
        return this.totalDuringTime;
    }

    public String getLrc() {
        List<Map.Entry<Long, String>> entries = new ArrayList<>(this.lyrics.entrySet());
        entries.sort(Map.Entry.comparingByKey(Comparator.reverseOrder()));
        for (Map.Entry<Long, String> timeLyrics : entries) {
            Long time = timeLyrics.getKey();
            if (time <= this.nowSongDuringTime) {
                return timeLyrics.getValue();
            }
        }
        return "";
    }

    public boolean isDone() {
        return this.nowSongDuringTime >= this.totalDuringTime;
    }
}
